<?php

/*
 * Base Class used for the primary resource APIs: backups, alerts, jobs, etc.
 */
abstract class BaseAPI {
    const DATE_TIME_FORMAT_US   = "m/d/Y h:i:s a";
    const MAX_OUTPUT = 1024;

    const UNNAMED_CREDENTIALS = "(Unnamed)";
    const NO_CREDENTIALS = "None";

    protected $conn;        // connection to database
    protected $Log;         // Logger

    public function __construct($conn, $Log){
        $this->conn = $conn;
        $this->Log = $Log;
        $this->Log->logDebug('Creating ' . get_class($this) . ' class.');
    }

    public function __destruct(){
        if (isset($this->Log)) {
            $this->Log->logDebug('Destroying ' . get_class($this) . ' class.');
        }
    }

    /*
     * For APIs which need an asset in the data, this routine checks to see if valid.
     * If an asset tag is found that is valid, the device_id is returned.
     */
    protected function validateAsset($data, &$result, &$device_id) {
        $valid = true;
        if (!isset($data['asset_tag']) || $data['asset_tag'] === false){
            $message = "No asset_tag found in data.";
            $this->Log->logError($message);
            $result = array(
                'result' => false,
                'message' => $message
            );
            $valid = false;
        } else {
            $device_id = $this->getDeviceID($data['asset_tag']);
            if ($device_id == -1) {
                $message = "Device ID not found for asset tag: ". $data['asset_tag'];
                $this->Log->logError($message);
                $result = array(
                    'result' => false,
                    'message' => $message
                );
                $valid = false;
            }
        }
        return $valid;
    }


    public function setTimestamp($subsystem, $timestamp, $days, $device_id) {
        if ($this->conn !== null) {
            $subsystemNumber = is_string($subsystem) ? $this->getSubsystemNumber($subsystem) : $subsystem;
            $currentTimestamp = $this->getTimestamp($subsystemNumber, $device_id);
            if ($currentTimestamp !== false) {
                if ($currentTimestamp == null) {
                    $sQuery = "INSERT INTO mgportal.timestamps (device_id, subsystem, update_time, day_count) VALUES (%d, %d, %d, %d);";
                    $query = sprintf($sQuery, $device_id, $subsystemNumber, $timestamp, $days);
                } else {
                    $query = "UPDATE mgportal.timestamps SET update_time=" . $timestamp;
                    $query .= ", day_count=" . $days . " WHERE device_id=" . $device_id . " AND subsystem=" . $subsystemNumber . ";";
                }
                Database::db_query($this->conn, $this->Log, $query);
            } else {
                $this->Log->logInfo('timestamp for ' . $subsystem . ' not found');
            }
        }
    }

    private function getSubsystemNumber($subsystem) {
        $num = 0;
        switch ($subsystem) {
            case 'backups':
                $num = 0;
                break;
            case 'copies':
                $num = 1;
                break;
            case 'alerts':
                $num = 2;
                break;
            case 'assets':
                $num = 3;
                break;
            case 'jobs':
                $num = 4;
                break;
        }
        return $num;
    }

    public function getTimestamp($subsystem, $device_id) {
        $timestamp = false;
        if ($this->conn !== null) {
            $subsystemNumber = is_string($subsystem) ? $this->getSubsystemNumber($subsystem) : $subsystem;
            $query = "SELECT update_time from mgportal.timestamps WHERE device_id=" . $device_id . " AND subsystem=" . $subsystemNumber;
            $result = Database::db_query($this->conn, $this->Log, $query);
            if ($result !== false) {
                $row = Database::db_fetch_row($this->Log,  $result);
                if ($row == null || $row == false) {
                    $timestamp = null;
                } else {
                    $timestamp = (int)$row[0];
                }
            }
        }
        return $timestamp;
    }

    public function getTimestampsInfo($subsystem, $device_id) {
        $info = false;
        if ($this->conn !== null) {
            $subsystemNumber = is_string($subsystem) ? $this->getSubsystemNumber($subsystem) : $subsystem;
            $query = "SELECT update_time, day_count from mgportal.timestamps WHERE device_id=" . $device_id . " AND subsystem=" . $subsystemNumber;
            $result = Database::db_query($this->conn, $this->Log, $query);
            if ($result !== false) {
                $row = Database::db_fetch_row($this->Log,  $result);
                if ($row == null || $row == false) {
                    $timestamp = null;
                    $day_count = 0;
                } else {
                    $timestamp = (int)$row[0];
                    $day_count = (int)$row[1];
                }
            }
            $info = array('timestamp' => $timestamp, 'day_count' => $day_count);
        }
        return $info;
    }

    /*
     * Given the asset tag of a device, return its ID.
     */
    public function getDeviceID($asset_tag) {
        $device_id = -1;
        if ($this->conn !== null) {
            $query = "SELECT id FROM mgportal.devices WHERE unique_identifier='" . $asset_tag . "'";
            $response = Database::db_query($this->conn, $this->Log, $query);
            if ($response !== false) {
                $row = Database::db_fetch_row($this->Log,  $response);
                $device_id = $row[0];
            }
        }
        return $device_id;
    }

    /*
     * Given the id of a device, return its asset tag/UUID.
     */
    protected function getDeviceUUID($deviceID) {
        $deviceUUID = false;
        $query = "SELECT unique_identifier FROM mgportal.devices WHERE id=" . $deviceID;
        $deviceResult = Database::db_query($this->conn, $this->Log, $query);
        if ($deviceResult !== false) {
            $row = Database::db_fetch_row($this->Log,  $deviceResult);
            $deviceUUID = $row[0];
        }
        return $deviceUUID;
    }

    public function sendCommand($deviceId, $cmd) {
        $result = true;
        if ($deviceId != null) {
            $deviceUUID = $this->getDeviceUUID($deviceId);
            if ($deviceUUID !== false) {
                if ($cmd == "update" || $cmd == Configuration::CMD_GET_ALL) {
                    $result = $this->executeCommand($deviceUUID, Configuration::CMD_GET_ALL);
                } else if ($cmd == Configuration::CMD_OPEN_TUNNEL) {
                    $result = $this->executeCommand($deviceUUID, $cmd);
                } else if ($cmd == Configuration::CMD_CLOSE_TUNNEL) {
                    $result = $this->executeCommand($deviceUUID, $cmd);
                } else if ($cmd == Configuration::CMD_CHECK_TUNNEL) {
                    $result = $this->executeCommand($deviceUUID, $cmd);
                } else {
                    $result = array('result' => false, 'message' => 'Unknown command to send to networked appliance');
                }
            } else {
                $result = array('result' => false, 'message' => 'Error looking up asset information');
            }
        } else {
            $result = array('result' => false, 'message' => 'Device ID must be specified for data update');
        }
        return $result;
    }

    public function executeCommand($deviceId, $parameter) {
        $result = true;

        $path = BASE_DIR . BIN_DIR . 'to_device.php ';
        $command = $path . $deviceId . " " . $parameter;
        $this->Log->logInfo("Command to run: " . $command);
        exec($command, $outputArray, $returnValue);
        if (is_array($outputArray) && count($outputArray) > 0) {
            $output = $outputArray[0];
        } else {
            $output = null;
        }
        $decoded = json_decode($output, true);
        switch ($parameter) {
            case Configuration::CMD_GET_ALL:
                if ($decoded != NULL) {
                    $status = isset($decoded['success']) ? $decoded['success'] : false;
                    $message = isset($decoded['output']) ? $decoded['output'] : 'Unknown Server error.  Please check server logs for more information.';
                } else {
                    $status = false;
                    if (strlen($output) > BaseAPI::MAX_OUTPUT) {
                        $output = substr($output, 0, BaseAPI::MAX_OUTPUT);
                    }
                    $message = $output;
                }
                $result = array('result' => $status, 'message' => $message, $output = $decoded);
                break;
            case Configuration::CMD_OPEN_TUNNEL:
                $pos = strpos($output, "Creating tunnel: ");
                $host = "";
                if ($pos !== false) {
                    $startTunnel = $pos + strlen("Creating tunnel: ");
                    $substring = substr($output, $startTunnel);
                    $endpos = strpos($substring, ':');
                    $tunnel = substr($substring, 0, $endpos);
                    $host = $decoded['host'];
                } else {
                    // Propagate error message.
                    if ($decoded != NULL) {
                        if (isset($decoded['output'])) {
                            $output = $decoded['output'];
                        }
                    }
                }
                if (isset($tunnel)) {
                    $result = array('result' => true, 'output' => $output, 'tunnel' => $tunnel, 'host' => $host);
                } else {
                    $result = array('result' => false, 'message' => $output);
                }
                break;
            case Configuration::CMD_CLOSE_TUNNEL:
                $result = true;
                $pos = strpos($output, "Success");
                if ($pos === false) {
                    $result = array('result' => false, 'message' => 'Error closing tunnel.');
                }
                break;
            case Configuration::CMD_CHECK_TUNNEL:
                $decoded = json_decode($output, true);
                if (isset($decoded['output'])) {
                    $tunnel = (int)$decoded['output'];
                    $host = $decoded['host'];
                    // Check to see which ssh session is for this tunnel, returns a string of the form <localTunnel>:localhost:<tunnel>.
                    $command = "ps -C ssh -o args | awk '{if ($7 ~ $tunnel && $10 ~ mgportal && $10 ~ known_hosts) print $7}'";
                    //$this->Log->logDebug("check command: " . $command);
                    exec($command, $resultArray, $returnValue);
                    if (is_array($resultArray) && count($resultArray) > 0) {
                        $line = $resultArray[0];
                        $localTunnel = (int)$line;
                    } else {
                        $localTunnel = $tunnel + 1;
                    }
                    $result = array('result' => true, 'output' => $output, 'tunnel' => $tunnel, 'localTunnel' => $localTunnel, 'host' => $host);
                } else {
                    $result = array('result' => false, 'message' => $output, 'tunnel' => 0);
                }
                break;
        }
        return $result;
    }

    protected function valid_credentials($username, $password) {
        $valid = true;

        $passencoded = "'" . str_replace("'", "'\''", $password) . "'";
        $userencoded = "'" . str_replace("'", "'\''", $username) . "'";
        $cmd = "echo -e $passencoded | sudo -u $userencoded -S whoami 2>/dev/null";
        $res = trim(shell_exec($cmd));
        if ($res !== $username) {
            $valid = false;
        }

        return $valid;
    }

};

?>