<?php

const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const CONFIG_FILE = "mgportal.conf";
const API_LOG_FILE = "portal_api";

require_once("HTTP.php");
require_once("api-base.php");
require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');
require_once(BASE_DIR . BIN_DIR . 'commands.php');
require_once(BASE_DIR . BIN_DIR . "database.php");

$Log = new Logger(Logger::DEBUG);
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "server");
$Config->load_configuration();
$Log->setLogLevel($Config->getLogLevel());
$Log->setLogFile(API_LOG_FILE);
$http = new HTTP();

class Settings extends BaseAPI {
    const DEFAULT_LOCATION = 1;
    const FILENAME = '/usr/local/mgportal/conf.d/mgportal.conf';
    const JSON_PRETTY_PRINT = 128;      // option to json_encode, only if PHP 5.4 or higher

    public function __construct($conn, $Log) {
        parent::__construct($conn, $Log);
    }

    public function get($flat = false, $showPasswords = false){
        $settings = file_get_contents(Settings::FILENAME);
        $settingsJSON = json_decode($settings, true);
        if ($flat) {
            $flatResult = array();
            foreach ($settingsJSON as $area => $setting) {
                foreach ($setting as $name => $value) {
                    if (is_array($value)) {
                        // If an array, convert to a string of the form:
                        // key^value,key^value,key^value;key^value;
                        $value = implode(";", array_map(function($o) {
                            $str = '';
                            foreach ($o as $key => $value) {
                                $str .= $key . '^' . $value . ',';
                            }
                            // remove last comma
                            $str = substr($str,0,-1);
                            return $str;
                        }, $value));
                    } else if (strstr($name, "password") && !$showPasswords) {
                        // do not return passwords!
                        $len = strlen($value);
                        $value = str_repeat('*', $len);
                    }
                    $item = array('section' => $area,
                        'field' => $name,
                        'value' => $value);
                    $flatResult[] = $item;
                }
            }
            $result = array('settings' => $flatResult);
        } else {
            $result = array('settings' => $settingsJSON);
        }
        return $result;
    }

    /*
     * Save the modified setting that comes in by payload.
     * To do so, we load the original file into a flat array, then merge with the changed element.
     */
    public function save($data){
        // show flat and get passwords from original on save.
        $orig = $this->get(true, true);
        if (isset($orig['settings'])) {
            $origSettings = $orig['settings'];
            // wrap data in an array for merge with the original.
            $newSetting = array($data);
            $newSettings = array_merge($origSettings, $newSetting);
            $settingsJSON = $this->toSettingsJSON($newSettings);
            $formatted = $this->format_json($settingsJSON, false, 4);
            $result = file_put_contents(Settings::FILENAME, $formatted);
            if ($result !== false) {
                $result = true;
            }
        } else {
            $result = array('result' => false, 'message' => 'Error loading settings file to update values');
        }
        return $result;
    }

    /*
     * Convert the flat array to a JSON-friendly format, an associative array
     * with client and server sub-components.
     */
    private function toSettingsJSON($data) {
        $payload = array('client' => array(), 'server' => array());
        for ($i = 0; $i < count($data); $i++) {
            $obj = $data[$i];
            $section = $obj['section'];
            $field = $obj['field'];
            $value = $obj['value'];
            if (is_numeric($value)) {
                $value = (int)$value;
            } else if (strtolower($value) == 'true') {
                $value = true;
            } else if (strtolower($value) == 'false') {
                $value = false;
            }
            $payload[$section][$field] = $value;
        }
        $jsonPayload = json_encode($payload);
        return $jsonPayload;
    }

    function format_json($json, $html = false, $tabspaces = null)
    {
        $tabcount = 0;
        $result = '';
        $inquote = false;
        $ignorenext = false;

        if ($html) {
            $tab = str_repeat("&nbsp;", ($tabspaces == null ? 4 : $tabspaces));
            $newline = "<br/>";
        } else {
            $tab = ($tabspaces == null ? "\t" : str_repeat(" ", $tabspaces));
            $newline = "\n";
        }

        for($i = 0; $i < strlen($json); $i++) {
            $char = $json[$i];

            if ($ignorenext) {
                $result .= $char;
                $ignorenext = false;
            } else {
                switch($char) {
                    case ':':
                        $result .= $char . (!$inquote ? " " : "");
                        break;
                    case '[':
                    case '{':
                        if (!$inquote) {
                            $tabcount++;
                            $result .= $char . $newline . str_repeat($tab, $tabcount);
                        }
                        else {
                            $result .= $char;
                        }
                        break;
                    case ']':
                    case '}':
                        if (!$inquote) {
                            $tabcount--;
                            $result = trim($result) . $newline . str_repeat($tab, $tabcount) . $char;
                        }
                        else {
                            $result .= $char;
                        }
                        break;
                    case ',':
                        if (!$inquote) {
                            $result .= $char . $newline . str_repeat($tab, $tabcount);
                        }
                        else {
                            $result .= $char;
                        }
                        break;
                    case '"':
                        $inquote = !$inquote;
                        $result .= $char;
                        break;
                    case '\\':
                        if ($inquote) $ignorenext = true;
                        $result .= $char;
                        break;
                    default:
                        $result .= $char;
                }
            }
        }

        // add newline
        $result .= $newline;

        return $result;
    }

    /*
     * Delete the specified setting.
     * To do so, we load the original file into a flat array, then merge with the changed element.
     */
    public function delete($data){
        // show flat and get passwords from original on delete.
        $orig = $this->get(true, true);
        if (isset($orig['settings'])) {
            $origSettings = $orig['settings'];
            $newSettings = array();
            for ($i = 0; $i < count($origSettings); $i++) {
                $origSetting = $origSettings[$i];
                if ($origSetting['section'] === $data['section'] && $origSetting['field'] === $data['field']) {
                    continue;
                } else {
                    $newSettings[] = $origSetting;
                }
            }
            $settingsJSON = $this->toSettingsJSON($newSettings);
            $formatted = $this->format_json($settingsJSON, false, 4);
            $result = file_put_contents(Settings::FILENAME, $formatted);
            if ($result !== false) {
                $result = true;
            }
        } else {
            $result = array('result' => false, 'message' => 'Error loading settings file to delete setting.');
        }
        return $result;
    }

};

$authMessage = HTTP::UNAUTHENTICATED_MESSAGE;
if ($http->isAuthenticated($authMessage)) {
    $db = new Database();
    $conn = $db->connect();
    if ($conn !== false) {
        $settings = new Settings($conn, $Log);
        $which = isset($_GET['location_id']) ? $_GET['location_id'] : null;
        switch ($http->getMethod()) {
            case 'get':
                $flat = isset($_GET['type']) && $_GET['type'] == "flat";
                $showPasswords = false;
                if (isset($_GET['passwords'])) {
                    if ($_GET['passwords'] === 'true') {
                        $showPasswords = true;
                    }
                }
                $result = $settings->get($flat, $showPasswords);
                $http->buildResult($result, "get");
                break;
            case 'put':
                $result = $settings->save($http->getData());
                $http->buildResult($result, "put");
                break;
            case 'delete':
                $result = $settings->delete($http->getData());
                $http->buildResult($result, "delete");
                break;
        }
        $db->closeConnection();
    } else {
        $http->buildFailure(false, "Error connecting to database");
    }
} else {
    $Log->logError($authMessage);
    $http->buildFailure(false, $authMessage);
}


?>